@@ -3,11 +3,12 @@ from typing import Dict, List
 from simple_ddl_parser import tokens as tok
 from simple_ddl_parser.dialects.hql import HQL
 from simple_ddl_parser.dialects.oracle import Oracle
+from simple_ddl_parser.dialects.redshift import Redshift
 from simple_ddl_parser.dialects.sql import BaseSQL
 from simple_ddl_parser.parser import Parser
 
 
-class DDLParser(Parser, BaseSQL, HQL, Oracle):
+class DDLParser(Parser, BaseSQL, HQL, Oracle, Redshift):
     """
     lex and yacc parser for parse ddl into BQ schemas
     """
@@ -92,7 +93,7 @@ class DDLParser(Parser, BaseSQL, HQL, Oracle):
             if t.type == "RP" and self.lexer.lp_open:
                 self.lexer.lp_open -= 1
             self.lexer.last_par = t.type
-        elif t.type == "TYPE" or t.type == "DOMAIN":
+        elif t.type == "TYPE" or t.type == "DOMAIN" or t.type == "TABLESPACE":
             self.lexer.is_table = False
         elif t.type == "TABLE" or t.type == "INDEX":
             self.lexer.is_table = True
